<?php
/**
* Copyright (C) 2017-2019 Petr Hucik <petr@getdatakick.com>
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@getdatakick.com so we can send you a copy immediately.
*
* @author    Petr Hucik <petr@getdatakick.com>
* @copyright 2017-2019 Petr Hucik
* @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/

require_once __DIR__ . '/classes/install.php';
require_once __DIR__ . '/classes/settings.php';
require_once __DIR__ . '/app-translation.php';
require_once __DIR__ . '/bootstrap.php';

use AdvDebug\Bootstrap;
use AdvDebug\Install;

class AdvDebug extends Module
{
    private $settings;

    public function __construct()
    {
        $this->name = 'advdebug';
        $this->tab = 'administration';
        $this->version = '1.0.0';
        $this->author = 'DataKick';
        $this->need_instance = 0;
        $this->bootstrap = true;
        parent::__construct();
        $this->displayName = $this->l('Advanced Debug Mode');
        $this->description = $this->l('Advanced Debug Mode');
        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => '1.6.1.999');
        $this->controllers = ['api'];
    }

    public function install($full = true)
    {
        return (
            parent::install() &&
            $this->installTab() &&
            $this->registerHook('displayHeader') &&
            Install::install() &&
            $this->getSettings()->init()
        );
    }

    public function uninstall($full = true)
    {
        return (
            Install::uninstall() &&
            $this->unregisterHook('displayHeader') &&
            $this->removeTab() &&
            parent::uninstall()
        );
    }

    public function reset()
    {
        return (
            $this->uninstall() &&
            $this->install()
        );
    }

    public function hookDisplayHeader()
    {
        if (Bootstrap::getInstance()->isEnabled()) {
            Media::addJsDef([
                'advDebugUrl' => $this->getUrl('api')
            ]);
            $this->context->controller->addJS($this->getSettings()->getAppUrl($this));
        }
    }

    public function getPath($relative)
    {
        $uri = rtrim($this->getPathUri(), '/');
        $rel = ltrim($relative, '/');
        return "$uri/$rel";
    }

    public function getUrl($controller, $params = [])
    {
        return $this->context->link->getModuleLink($this->name, $controller, $params);
    }

    public function getContent()
    {
        Tools::redirectAdmin($this->context->link->getAdminLink('AdminAdvDebugBackend'));
    }

    public function getSettings()
    {
        if (is_null($this->settings)) {
            $this->settings = new \AdvDebug\Settings();
        }
        return $this->settings;
    }

    public function getFrontTranslations()
    {
        $transl = new \AdvDebug\AppTranslation($this);
        return $transl->getFrontTranslations();
    }

    public function getBackTranslations()
    {
        $transl = new \AdvDebug\AppTranslation($this);
        return $transl->getBackTranslations();
    }

    private function installTab()
    {
        $tab = new Tab();
        $tab->active = 1;
        $tab->class_name = 'AdminAdvDebugBackend';
        $tab->module = $this->name;
        $tab->id_parent = $this->getTabParent();
        $tab->name = array();
        foreach (Language::getLanguages(true) as $lang) {
            $tab->name[$lang['id_lang']] = 'Advanced Debug Mode';
        }
        return $tab->add();
    }

    private function removeTab()
    {
        $tabId = Tab::getIdFromClassName('AdminAdvDebugBackend');
        if ($tabId) {
            $tab = new Tab($tabId);
            return $tab->delete();
        }
        return true;
    }

    private function getTabParent()
    {
        $parent = Tab::getIdFromClassName('AdminTools');
        if ($parent !== false) {
            return $parent;
        }
        return 0;
    }

}
