<?php
/**
* Copyright (C) 2017-2019 Petr Hucik <petr@getdatakick.com>
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@getdatakick.com so we can send you a copy immediately.
*
* @author    Petr Hucik <petr@getdatakick.com>
* @copyright 2017-2019 Petr Hucik
* @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/

namespace AdvDebug;

use \Exception;
use \Module;

class Bootstrap
{
    private static $instance;

    private $debugEnabled = false;
    private $moduleEnabled = null;
    private $key;

    private function __construct($key)
    {
        $this->key = $key;
    }

    public static function createInstance($asciiKey)
    {
        static::$instance = new Bootstrap($asciiKey);
        return static::$instance;
    }

    public static function getInstance()
    {
        if (is_null(static::$instance)) {
            static::$instance = new Bootstrap('');
        }
        return static::$instance;
    }


    public function isEnabled()
    {
        if (!$this->debugEnabled) {
            return false;
        }

        if (is_null($this->moduleEnabled)) {
            $this->moduleEnabled = Module::isInstalled('advdebug') && Module::isEnabled('advdebug');
        }
        return $this->moduleEnabled;
    }

    public function bootstrap()
    {
        if (!defined('_PS_MODE_DEV_')) {
            $this->debugEnabled = $this->checkCookie();
            define('_PS_MODE_DEV_', $this->debugEnabled);
        }
    }

    private function checkCookie()
    {
        if (isset($_COOKIE['advdebug'])) {
            $value = $this->decrypt($_COOKIE['advdebug']);
            if ($value) {
                return true;
            }
            unset($_COOKIE['advdebug']);
            setcookie('advdebug', '', time() - 3600000, '/');
            return false;
        }
        return false;
    }

    public function setCookie($validity = 3600)
    {
        $now = time();
        $cookie = $this->encrypt(json_encode([
            'c' => $now,
            'l' => $validity
        ]));
        setcookie('advdebug', $cookie, $now + $validity, '/');
    }

    private function encrypt($plaintext)
    {
        $ivsize = openssl_cipher_iv_length('aes-256-cbc');
        $iv = openssl_random_pseudo_bytes($ivsize);
        $ciphertext = openssl_encrypt($plaintext, 'aes-256-cbc', $this->key, OPENSSL_RAW_DATA, $iv);
        return $this->base64Encode($iv . $ciphertext);
    }

    private function decrypt($encoded)
    {
        $ciphertext = $this->base64Decode($encoded);
        $ivsize = openssl_cipher_iv_length('aes-256-cbc');
        $iv = mb_substr($ciphertext, 0, $ivsize, '8bit');
        $ciphertext = mb_substr($ciphertext, $ivsize, null, '8bit');
        return openssl_decrypt($ciphertext, 'aes-256-cbc', $this->key, OPENSSL_RAW_DATA, $iv);
    }

    private function base64Encode($data)
    {
        return rtrim(strtr(base64_encode($data), '+/', '-_'), '=');
    }

    private function base64Decode($data)
    {
        return base64_decode(str_pad(strtr($data, '-_', '+/'), strlen($data) % 4, '=', STR_PAD_RIGHT));
    }

}
