<?php
/**
* Copyright (C) 2017-2019 Petr Hucik <petr@getdatakick.com>
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@getdatakick.com so we can send you a copy immediately.
*
* @author    Petr Hucik <petr@getdatakick.com>
* @copyright 2017-2019 Petr Hucik
* @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/

namespace AdvDebug;

class Install
{

    public static function install()
    {
        if (!extension_loaded('openssl') || !function_exists('openssl_encrypt')) {
            return false;
        }
        $key = \Tools::passwdGen(32);

        $customDefines = _PS_CONFIG_DIR_ . 'defines_custom.inc.php';
        $original = null;
        if (file_exists($customDefines)) {
            $original = file_get_contents($customDefines);
            $original = static::removePhpTag($original);
            $original = static::removeBoostrapCode($original);
        }
        $content = static::getCustomDefinesContent($original, $key);
        return file_put_contents($customDefines, $content) !== false;
    }

    public static function uninstall()
    {
        $customDefines = _PS_CONFIG_DIR_ . 'defines_custom.inc.php';
        if (file_exists($customDefines)) {
            $content = file_get_contents($customDefines);
            $content = static::removeBoostrapCode($content);
            if (static::removePhpTag($content) === "") {
                return unlink($customDefines);
            } else {
                return file_put_contents($customDefines, $content) !== false;
            }
        }
        return true;
    }

    private static function removeBoostrapCode($code)
    {
        return preg_replace("#\s*\/\/ ADVDEBUG_BEGIN_MARKER.*ADVDEBUG_END_MARKER\s*#s", "\n\n", $code);
    }

    private static function removePhpTag($content)
    {
        return preg_replace("#\<\?php\s*#", "", $content);
    }

    private static function getCustomDefinesContent($original, $key)
    {
        $ret = "<?php\n\n";
        $ret .= "// ADVDEBUG_BEGIN_MARKER\n";
        $ret .= "if (! class_exists('\AdvDebug\Bootstrap') && file_exists(__DIR__ . '/../modules/advdebug/bootstrap.php')) {\n";
        $ret .= "  include_once(__DIR__ . '/../modules/advdebug/bootstrap.php');\n";
        $ret .= "  \AdvDebug\Bootstrap::createInstance('$key')->bootstrap();\n";
        $ret .= "}\n";
        $ret .= "// ADVDEBUG_END_MARKER\n\n";
        if ($original) {
            $ret .= $original;
        }
        return $ret;
    }
}
