<?php
/**
* Copyright (C) 2017-2019 Petr Hucik <petr@getdatakick.com>
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@getdatakick.com so we can send you a copy immediately.
*
* @author    Petr Hucik <petr@getdatakick.com>
* @copyright 2017-2019 Petr Hucik
* @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/

class AdminAdvDebugBackendController extends ModuleAdminController
{
    public $module;

    public function __construct()
    {
        parent::__construct();
        $this->display = 'view';
        $this->bootstrap = false;
        $this->addCSS($this->getPath('views/css/back.css'));
        $this->addCSS($this->getPath('views/css/AdvDebug.css'));
        $this->addJs($this->module->getSettings()->getBackendAppUrl($this->module));
    }

    public function display()
    {
        $settings = $this->module->getSettings();
        $this->display_footer = false;
        $platform = 'prestashop';
        $platformVersion = _PS_VERSION_;
        if (defined('_TB_VERSION_')) {
            $platform = 'thirtybees';
            $platformVersion = _TB_VERSION_;
        }
        $link = $this->context->link;

        $links = [];
        foreach (Shop::getShops() as $shop) {
            $links[] = [
                'name' => $shop['name'],
                'link' => $link->getModuleLink($this->module->name, 'api', ['enable-debug' => 'REPLACECODE'], null, null, $shop['id_shop'])
            ];
        }

        $this->context->smarty->assign([
            'help_link' => null,
            'title' => $this->l('Advanced Debug Mode'),
            'advDebug' => [
                'apiUrl' => $this->context->link->getAdminLink('AdminAdvDebugBackend'),
                'licenseUrl' => $settings->getApiUrl(),
                'activated' => $settings->isActivated(),
                'version' => $this->module->version,
                'platform' => $platform,
                'platformVersion' => $platformVersion,
                'versionCheck' => $settings->getCheckModuleVersion(),
                'settings' => $settings->get(),
                'links' => $links,
                'translations' => $this->module->getBackTranslations(),
            ]
        ]);
        return parent::display();
    }

    public function createTemplate($tpl_name)
    {
        if ($this->viewAccess() && $tpl_name === 'content.tpl') {
            $path = _PS_MODULE_DIR_ . $this->module->name . '/views/templates/admin/backend.tpl';
            return $this->context->smarty->createTemplate($path, $this->context->smarty);
        }
        return parent::createTemplate($tpl_name);
    }

    private function getPath($path)
    {
        return $this->module->getPath($path);
    }

    public function ajaxProcessCommand()
    {
        $moduleInstance = $this->module;
        $error = null;
        $result = null;
        try {
            $result = $this->dispatchCommand(Tools::getValue('cmd'));
        } catch (Exception $e) {
            $error = $e->getMessage();
        }
        $this->reply($error, $result);
        die();
    }

    private function dispatchCommand($cmd)
    {
        $payload = json_decode(Tools::getValue('payload'), true);
        switch ($cmd) {
            case 'setLatestVersion':
                return $this->setLatestVersion($payload);
            case 'setSettings':
                return $this->setSettings($payload);
            case 'accountActivated':
                return $this->accountActivated($payload);
            default:
                throw new Exception("Unknown command $cmd");
        }
    }


    private function setLatestVersion($data)
    {
        if (!isset($data['version'])) {
            throw new Exception('Version not set');
        }
        if (!isset($data['ts'])) {
            throw new Exception('Timestamp not set');
        }
        $this->module->getSettings()->setCheckModuleVersion($data['version'], $data['ts'], $data['notes']);
        return true;
    }

    private function setSettings($data)
    {
        if (!isset($data['settings'])) {
            throw new Exception('Settings not set');
        }
        $this->module->getSettings()->set($data['settings']);
        return true;
    }

    private function accountActivated($data)
    {
        $this->module->getSettings()->setActivated();
        return true;
    }

    private function reply($error, $result)
    {
        if ($error) {
            echo json_encode(['success' => false, 'error' => $error]);
        } else {
            echo json_encode(['success' => true, 'result' => $result]);
        }
    }

    private function getAdminLink($controller, $params)
    {
        $link = $this->context->link;
        $idLang = $this->context->language->id;
        $params = array_merge($params, ['token' => Tools::getAdminTokenLite($controller)]);
        return Dispatcher::getInstance()->createUrl($controller, $idLang, $params, false);
    }

}
