<?php
/**
* Copyright (C) 2017-2019 Petr Hucik <petr@getdatakick.com>
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@getdatakick.com so we can send you a copy immediately.
*
* @author    Petr Hucik <petr@getdatakick.com>
* @copyright 2017-2019 Petr Hucik
* @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/

use AdvDebug\Bootstrap;

class AdvDebugApiModuleFrontController extends ModuleFrontController
{
    public $module;

    public function __construct()
    {
        parent::__construct();
        $this->context = Context::getContext();
    }

    public function initContent()
    {
        parent::initContent();
        if (Tools::isSubmit('action') && Tools::getValue('action') == 'command') {
            $this->ajaxProcessCommand();
        } else {
            if (Tools::getValue('enable-debug')) {
                $this->initDebugMode(Tools::getValue('enable-debug'));
            }
            Tools::redirect('index.php');
        }
    }

    public function ajaxProcessCommand()
    {
        $error = null;
        $errorCode = -1;
        $result = null;
        try {
            $cmd = Tools::getValue('cmd');
            $result = $this->dispatchCommand($cmd);
        } catch (Exception $e) {
            $error = $e->getMessage();
            $errorCode = $e->getCode();
        }
        $this->reply($error, $errorCode, $result);
    }

    private function dispatchCommand($cmd)
    {
        switch ($cmd) {
            case 'initDebugMode':
                return $this->initDebugMode(Tools::getValue('code'));
            default:
                throw new Exception("Unknown command $cmd");
        }
    }

    private function initDebugMode($code)
    {
        $profiles = $this->module->getSettings()->getProfiles();
        if (isset($profiles[$code])) {
            $profile = $profiles[$code];
            $expiration = $profile['expiration'];
            return Bootstrap::getInstance()->setCookie($expiration);
        } else {
            die('Debug profile not found');
        }
    }

    private function reply($error, $errorCode, $result)
    {
        if ($error) {
            die(json_encode(['success' => false, 'error' => $error, 'errorCode' => $errorCode]));
        } else {
            die(json_encode(['success' => true, 'result' => $result]));
        }
    }
}
